/**
 * Dialog box for editing individual Finch Actions.
 * 
 * <P>
 * Approach:
 * <UL>
 * <LI> The dialog box will contain components for all possible values 
 * for any action.
 * <LI> Depending on the selected action type (via the Combo Box), a subset 
 * of these components will be displayed (this is configured using the 
 * setVisible() method for any graphics component).
 * </UL>
 * 
 * <P>
 * @author Andrew H. Fagg
 * 
 */

import javax.swing.JColorChooser;
import javax.swing.JDialog;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.JFileChooser.*;
import javax.swing.filechooser.*;
import javax.swing.event.*;
import javax.swing.border.*;

abstract public class FinchActionDialogAbstract extends JDialog {
	
	// Panel that contains a label and a combo box to represent the different 
	//  types of Finch Actions.
	protected JComboBox aComboBox = new JComboBox(FinchActionType.getActionList());

	protected JLabel lComboBox = new JLabel("Action Type: ");
	protected JPanel pComboBox = new JPanel();
	
	
	// Panel that contains the label and the text field for the name of the action
	protected JTextField aName = new JTextField("", 15);
	protected JLabel lName = new JLabel("Name: ");
	protected JPanel pName = new JPanel();
	
	// Panel that contains the label and slider for the duration
	protected JSlider sPriority = new JSlider(0, 30, 10);
	protected JLabel lPriority = new JLabel("Priority: ");
	protected JPanel pPriority = new JPanel();
	
	// Panel that contains the label and slider for the duration
	protected JSlider sDuration = new JSlider(0, 10000, 1000);
	protected JLabel lDuration = new JLabel("Duration (ms): ");
	protected JPanel pDuration = new JPanel();
	
	// Panel containing the buttons at the bottom of the dialog box
	protected JPanel jpButtons = new JPanel();
	protected JButton jbtOK = new JButton("OK");
	protected JButton jbtCancel = new JButton("Cancel");
	
	///////
	// Finch action that corresponds to the values entered into the 
	//  dialog box.  This field is filled in as the dialog box is 
	//  closing.  A value of null indicates no action has been specified
	//  (e.g., in the case of a 'Cancel' action).
	protected FinchAction action;
	
	// The nose color for the FlashBeak Action
	protected Color noseColor = new Color(0,0,0);
	///////////////////////////////////////////////////////
	
	/**
	 * Constructor
	 * 
	 * <P>
	 * Creates:
	 *  - action type combo box
	 *  - action name
	 *  - priority slider
	 *  - duration slider
	 * 
	 */
	public FinchActionDialogAbstract(){
		// Call the JDialog constructor
		super((java.awt.Frame) null, true);
		
		// Configure the look of the dialog box
		this.setTitle("Edit Finch Action");
		this.setLayout(new GridLayout(0,1,10,10));
		
		//////
		// First row: combo box for selecting action type
		
		// Label
		pComboBox.add(lComboBox);
		
		// Combo box
		pComboBox.add(aComboBox);
		
		// Add this panel to the dialog frame
		this.add(pComboBox);
		
		// When the combo box state changes, update the fields that are visible
		aComboBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e){
				reconfigureDialog();
			}
		});
		
		//////
		// Second row: Text field containing the name of the Finch Action
		
		// Label
		pName.add(lName);
		
		// The actual text field
		pName.add(aName);
		
		// Add the Name  panel to the dialog frame
		this.add(pName);

		////////////////////////
		// Priority row
		// Label
		pPriority.add(lPriority);
		
		// The slider
		pPriority.add(sPriority);
		sPriority.setPaintLabels(true);
		sPriority.setPaintTicks(true);
		sPriority.setMajorTickSpacing(10);
		sPriority.setMinorTickSpacing(1);
		
		// Add this row to the frame
		this.add(pPriority);
		

		////////////////////////
		// Duration row
		// Label
		pDuration.add(lDuration);
		
		// The slider
		pDuration.add(sDuration);
		sDuration.setPaintLabels(true);
		sDuration.setPaintTicks(true);
		sDuration.setMajorTickSpacing(2500);
		sDuration.setMinorTickSpacing(250);
		
		// Add this row to the frame
		this.add(pDuration);
		
		////////////////////////
		// Action listeners for the Buttons.  Note: 
		//  the buttons have not been added to the frame
		
		// OK Button
		jbtOK.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e){
				// Create the action that corresponds to the values in the dialog box
				createNewAction();
				// Close the dialog box
				setVisible(false);
			}
		});
		
		// Cancel button
		jbtCancel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e){
				//  Close the dialog box
				setVisible(false);
			}
		});
		
		// Add the buttons to the panel
		jpButtons.add(jbtOK);
		jpButtons.add(jbtCancel);
		
		/////////////////////////////////////////////////////////////
	}
	
	/**
	 * Given a FinchAction, configure the dialog box values
	 *   to reflect this action.
	 *   <P>
	 *   
	 * <P>
	 * Algorithm:
	 * <OL>
	 * <LI> Set all of the dialog box values to reasonable defaults
	 * <LI> For the specified action, copy the values from the action
	 * into the relevant dialog box values.  Note that any action type 
	 * will only specify a subset of the dialog box values.
	 * </OL>
	 * 
	 *  @param action A FinchAction or null
	 * 
	 */
	abstract public void setAction(FinchAction action);
	
	
	/**
	 * Create a FinchAction that corresponds to the current configuration
	 * of the dialog box.
	 * 
	 * <P>
	 * 
	 * Postconditions:
	 * <UL>
	 * <LI> this.action is set to the new FinchAction
	 * </UL>
	 * 
	 * <P>
	 * 
	 * Algorithm:
	 * <UL>
	 * <LI> Read the Finch Action type from the combo box
	 * <LI> For each type, call the appropriate Finch Action constructor
	 * </UL>
	 *
	 */
	abstract protected void createNewAction();
	
	
	////////////////////////////////////
	/**
	 * Accessor to action
	 * 
	 * @return The new FinchAction that has been created by the dialog box
	 *                (may be null)
	 */
	public FinchAction getAction(){
		return(action);
	}
	
	
	/**
	 * Reconfigure the look of the dialog box to show only the components that
	 * correspond to the Finch Action type that is selected by the combo box.
	 * 
	 * <P>
	 * Algorithm:
	 * <OL>
	 * <LI> For all property values that are not part of all Finch Action types,
	 * turn off the visibility (using the setVisible() method for the panels).
	 * <LI> For the currently selected Finch Action type (determined by the combo
	 * box), turn the visibility on for the necessary components.
	 * <LI> If the action type is FinchNose, then set the color of the component
	 * that indicates the currently selected color of this.noseColor
	 * </OL>
	 *
	 */
	abstract protected void reconfigureDialog();
	
	

	/**
	 * Open the color chooser dialog box to specify the nose color
	 * 
	 * <P>
	 * 
	 * Algorithm:
	 * <OL>
	 * <LI> Open a color chooser dialog box.
	 * <LI> If the returned color is valid, then copy this color to
	 * this.noseColor
	 * </OL>
	 */
	abstract protected void colorChooser();
	
}
